/*
 * Copyright (c) 1995 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Trevor Blackwell
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie the
 * rights to redistribute these changes.
 */
/*
 * Modified to handle memset() by David Mosberger-Tang.
 */
#ifdef MEMSET
# define ADRREG		$16
# define VALREG		$17
# define LENREG		$18
# define FUNCTION	memset
# define LONGIFY(r)
# define MS(i,a1,a2)	i,a1,a2
# define BZ(i,a1,a2)
#else
# define ADRREG		$16
# define VALREG		$31
# define LENREG		$17
# define FUNCTION	bzero
# define LONGIFY(r)	addl r,$31,r
# define MS(i,a1,a2)
# define BZ(i,a1,a2)	i,a1,a2
#endif

	.set	noreorder
	.set	noat

	.globl FUNCTION
	.ent FUNCTION
FUNCTION:
	.frame $30, 0, $26
	.mask 0, 0
	.fmask 0, 0
	.prologue 0

BZ(	addl	LENREG,$31,LENREG)	/* convert integer count to long */

	/* replicate 8-bit value 8 times: */
MS(	zapnot	VALREG,0x01,VALREG)
MS(	bis	ADRREG,$31,$0)		/* setup return value */
MS(	sll	VALREG,8,$2)
	bic	LENREG,63,$4		/* $4 <- (LEN / 64) * 64 */
MS(	bis	VALREG,$2,VALREG)
MS(	sll	VALREG,16,$2)
	subq	$31,ADRREG,$1
MS(	bis	VALREG,$2,VALREG)
MS(	sll	VALREG,32,$2)
	and	$1,0x07,$1	/* $1 <- (0-size)%8 */
MS(	bis	VALREG,$2,VALREG)

#ifndef MEMSET
	ble	LENREG,done
#endif

	beq	$1,aligned	/* address is long-aligned -> */
	cmpule	LENREG,$1,$2
	blbc	$2,firstpartial	/* LEN > # of bytes in first quadword -> */

	/*
	 * All bytes are contained in the first partial word:
	 */
	ldq_u	$3,0($16)
	lda	$1,-1($31)
	mskql	$1,LENREG,$1	/* create mask with LENREG 0xff bytes */
	insql	$1,ADRREG,$1	/* bring mask into position */
	bic	$3,$1,$3	/* clear masked bits from loaded word */
MS(	and	VALREG,$1,VALREG)	/* clear non-masked bits from VALREG */
MS(	bis	VALREG,$3,$3)	/* or the two together */
	stq_u	$3,0(ADRREG)
	ret	$31,($26),0

firstpartial:
	/* Handle the first partial word */
	ldq_u	$3,0(ADRREG)
MS(	insql	VALREG,ADRREG,$5)
	subq	LENREG,$1,LENREG
	mskql	$3,ADRREG,$3		/* set bytes (ADRREG%8)..7 in long */
	bic	LENREG,63,$4		/* $4 <- (LENREG / 64) * 64 */
MS(	bis	$5,$3,$3)
	stq_u	$3,0(ADRREG)

	addq	ADRREG,$1,ADRREG	/* round ADRREG up to next word */

aligned:
	/*
	 * Loop, setting 64 bytes at a time:
	 */
	beq	$4,loop_done

1:	stq	VALREG,0x00(ADRREG)
	stq	VALREG,0x08(ADRREG)
	stq	VALREG,0x10(ADRREG)
	stq	VALREG,0x18(ADRREG)
	subq	$4,64,$4
	stq	VALREG,0x20(ADRREG)
	stq	VALREG,0x28(ADRREG)
	stq	VALREG,0x30(ADRREG)
	stq	VALREG,0x38(ADRREG)
	addq	ADRREG,0x40,ADRREG
	bne	$4,1b

loop_done:
	/*
	 * Handle the last 0..7 longs:
	 */
	and	LENREG,0x38,$5
	and	LENREG,0x07,$6		/* 0..7 bytes left */
	beq	$5,lastpartial

2:	stq	VALREG,0(ADRREG)
	subq	$5,8,$5
	addq	ADRREG,8,ADRREG
	bne	$5,2b

	/* Handle the last partial word: */
lastpartial:
	beq	$6,done		/* don't touch more words than necessary */

	ldq	$1,0(ADRREG)
MS(	mskql	VALREG,$6,VALREG)
	mskqh	$1,$6,$1
MS(	bis	VALREG,$1,$1)
	stq	$1,0(ADRREG)

done:	ret	$31,($26),0

	.end FUNCTION
