/* syscall.S - generalized linux system call interface */

#include <alpha/regdef.h>
#include <asm/pal.h>

/*
 * This is for COMPATIBILITY with Linux/x86 only.  Linux/Alpha system
 * calls return an error indication in a3.  This allows to return
 * arbitrary 64bit values in v0 (because negative values are not
 * mistaken as error numbers).  However, C allows to return only one
 * value so the interface below folds the error indication passed in
 * a3 back into v0: it sets v0 to -errno if an error occurs.  Thus,
 * no negative 64bit numbers can be returned.  To avoid this problem,
 * use assembly stubs wherever possible/convenient.
 *
 * Usage:
 *
 * int	syscall(syscall_number, arg1, arg2, arg3, arg4, arg5)
 *
 * syscall_number = the index of the system call we're invoking
 * arg1-arg5 = up to 5 integer arguments to the system call
 *
 * We need to do some arg shifting: the kernel expects the
 * syscall number in v0 and the first five args in a0-a4.
 *
 */
	.set	noat

	.weakext syscall,  __syscall
	.weakext syscall0, __syscall0
	.weakext syscall1, __syscall1
	.weakext syscall2, __syscall2
	.weakext syscall3, __syscall3
	.weakext syscall4, __syscall4
	.weakext syscall5, __syscall5

	.globl	__syscall
	.ent	__syscall
__syscall:
	.frame	sp,0,ra,0
	ldgp	gp,0(t12)	# Make sure gp points somewhere sane
	.prologue 1
	bis	a0,a0,v0	# Syscall number -> v0
	bis	a1,a1,a0	# arg1-arg5 -> a0-a4
	bis	a2,a2,a1
	bis	a3,a3,a2
	bis	a4,a4,a3
	bis	a5,a5,a4

	call_pal PAL_callsys	# Invoke system call
	bne	a3,error_return	# br if a3=0 (no error)
	ret	zero,(ra)

	.end	__syscall


/* Functions called from the _syscallX macros.
 * Each call takes "x" number of args, followed by the syscall
 * number.  These routines are trivial dispatches.
 */

	.globl	__syscall0
	.ent	__syscall0
__syscall0:
	.frame	sp,0,ra,0
	.prologue 1
	bis		a0,a0,v0
	call_pal	PAL_callsys
	bne		a3,error_return
	ret		zero,(ra)

	.end	__syscall0

	.globl	__syscall1
	.ent	__syscall1
__syscall1:
	.frame	sp,0,ra,0
	.prologue 1
	bis		a1,a1,v0
	call_pal	PAL_callsys
	bne		a3,error_return
	ret		zero,(ra)
	.end	__syscall1

	.globl	__syscall2
	.ent	__syscall2
__syscall2:
	.frame	sp,0,ra,0
	.prologue 1
	bis		a2,a2,v0
	call_pal	PAL_callsys
	bne		a3,error_return
	ret		zero,(ra)
	.end	__syscall2

	.globl	__syscall3
	.ent	__syscall3
__syscall3:
	.frame	sp,0,ra,0
	.prologue 1
	bis		a3,a3,v0
	call_pal	PAL_callsys
	bne		a3,error_return
	ret		zero,(ra)
	.end	__syscall3

	.globl	__syscall4
	.ent	__syscall4
__syscall4:
	.frame	sp,0,ra,0
	.prologue 1
	bis		a4,a4,v0
	call_pal	PAL_callsys
	bne		a3,error_return
	ret		zero,(ra)
	.end	__syscall4

	.globl	__syscall5
	.ent	__syscall5
__syscall5:
	.frame	sp,0,ra,0
	.prologue 1
	bis		a5,a5,v0
	call_pal	PAL_callsys
	bne		a3,error_return
	ret		zero,(ra)
	.end	__syscall5


	.ent	error_return
error_return:
	.frame	sp,0,ra,0
	.prologue 1

2:	subq	zero,v0,v0	# and return -1 to caller
	ret	zero,(ra)

	.end	error_return
