/* This file was originally written by David Mosberger-Tang. Modified
   to fit in libc by Bob Manson. */

#include <fcntl.h>
#include <unistd.h>
#include <syscall.h>

#include <sys/mman.h>

#include <linux/config.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/fcntl.h>

#include <asm/io.h>
#include <asm/page.h>
#include <asm/system.h>

#define MAX_PORT	0x10000
#define vuip		volatile unsigned int *

#undef outb
#undef inb

/*
 * This needs to be cleaned up:
 */
#define IOSYS_ALPHA_JENSEN	1	/* Jensen (EISA bus) */
#define IOSYS_ALPHA_PCI		2	/* PCI I/O system (LCA & APECS) */

#if defined(CONFIG_ALPHA_NONAME)
# define CONFIG_IOSYS	IOSYS_ALPHA_PCI
# define IO_SHIFT	5
# define IO_BASE	LCA_IO
# define IO_TYPE_BYTE	0x00
# define IO_TYPE_WORD	0x08
# define IO_TYPE_LONG	0x18
#elif defined(CONFIG_ALPHA_CABRIOLET)
# define CONFIG_IOSYS	IOSYS_ALPHA_PCI
# define IO_SHIFT	5
# define IO_BASE	APECS_IO
# define IO_TYPE_BYTE	0x00
# define IO_TYPE_WORD	0x08
# define IO_TYPE_LONG	0x18
#elif defined(CONFIG_ALPHA_JENSEN)
# define CONFIG_IOSYS	IOSYS_ALPHA_JENSEN
# define IO_SHIFT	7
# define IO_BASE	EISA_IO
# define IO_TYPE_BYTE	0x00
# define IO_TYPE_WORD	0x20
# define IO_TYPE_LONG	0x60
#else
# error Unknown I/O system type.
#endif


static struct hae hae;

extern void __sethae (unsigned long new_hae);


inline void
_sethae(unsigned long addr)
{
#if CONFIG_IOSYS == IOSYS_ALPHA_JENSEN
	/* hae on the Jensen is bits 31:25 shifted right */
	addr >>= 25;
	if (addr != hae.cache)
		__sethae(addr);
#elif CONFIG_IOSYS == IOSYS_ALPHA_LCA
	unsigned long msb;

	/* no need to set hae if msb is 0: */
	msb = addr & 0xf8000000;
	if (msb && msb != hae.cache) {
		__sethae(msb);
	}
#endif
}


/*
 * I/O access is restricted to ISA port space (ports 0..65535).
 * Modern devices hopefully are sane enough not to put any performance
 * critical registers in i/o space.
 *
 * On the first call to ioperm(), the entire ISA port space is mapped
 * into the virtual address space at address uio_base.  mprotect()
 * calls are then used to enable/disable access to ports.  Per page,
 * there are PAGE_SIZE>>IO_SHIFT I/O ports (e.g., 256 ports on a
 * Low Cost Alpha based system using 8KB pages).
 *
 * Keep in mind that this code also has to run in a 32bit address space.
 * It is therefore unreasonable to expect mmap'ing the entire sparse
 * address space would work (e.g., the Low Cost Alpha chip has an I/O
 * address space that's 512MB large!).
 */
static unsigned long uio_base;

int
ioperm (unsigned long from, unsigned long num, int turn_on)
{
	unsigned long addr, len;
	int prot;

	/* this test isn't as silly as it may look like; consider overflows! */
	if (from >= MAX_PORT || from + num > MAX_PORT) {
		extern int errno;

		errno = EINVAL;
		return -1;
	}
	if (turn_on) {
		if (!uio_base) {
			int fd;

			hae.reg   = (unsigned long*) HAE_ADDRESS;
			hae.cache = 0;
			__sethae(hae.cache);	/* synchronize with hw */

			fd = open("/dev/mem", O_RDWR);
			if (fd < 0) {
				return fd;
			}
			uio_base = (unsigned long) mmap(0, MAX_PORT<<IO_SHIFT,
							PROT_NONE, MAP_SHARED,
							fd, IO_BASE);
			close(fd);
			if (uio_base == -1) {
				return -1;
			}
		}
		prot = PROT_READ | PROT_WRITE;
	} else {
		if (!uio_base) {
			return 0;	/* never was turned on... */
		}
		/* turnoff access to relevant pages: */
		prot = PROT_NONE;
	}
	addr  = uio_base + (from << IO_SHIFT);
	len   = (num << IO_SHIFT) + (addr & ~PAGE_MASK);
	addr &= PAGE_MASK;
	return mprotect((void *)addr, len, prot);
}

void
_outb(unsigned char b, unsigned long port)
{
	unsigned int w;
	unsigned long addr = (port << IO_SHIFT) + uio_base + IO_TYPE_BYTE;

	if (port >= MAX_PORT)
	  return;

	_sethae(0);
	asm ("insbl %2,%1,%0" : "r="(w) : "ri"(port & 0x3), "r"(b));
	*(vuip)addr = w;
	mb();
}

void
_outw(unsigned short b, unsigned long port)
{
	unsigned int w;
	unsigned long addr = (port << IO_SHIFT) + uio_base + IO_TYPE_WORD;

	if (port >= MAX_PORT)
	  return;

	_sethae(0);
	asm ("inswl %2,%1,%0" : "r="(w) : "ri"(port & 0x3), "r"(b));
	*(vuip)addr = w;
	mb();
}

void
_outl(unsigned int b, unsigned long port)
{
	unsigned long addr = (port << IO_SHIFT) + uio_base + IO_TYPE_LONG;

	if (port >= MAX_PORT)
	  return;

	_sethae(0);
	*(vuip)addr = b;
	mb();
}

unsigned int
_inb(unsigned long port)
{
	long result = *(vuip) ((port << IO_SHIFT) + uio_base + IO_TYPE_BYTE);

	_sethae(0);
	result >>= (port & 3) * 8;
	return 0xffUL & result;
}

unsigned int
_inw(unsigned long port)
{
	long result = *(vuip) ((port << IO_SHIFT) + uio_base + IO_TYPE_WORD);

	_sethae(0);
	result >>= (port & 3) * 8;
	return 0xffffUL & result;
}

unsigned int
_inl(unsigned long port)
{
	_sethae(0);
	return *(vuip) ((port << IO_SHIFT) + uio_base + IO_TYPE_LONG);
}
