/* Copyright (C) 1991 Free Software Foundation, Inc.
This file is part of the GNU C Library.

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */

#include <errno.h>

#include "mathimpl.h"

/* Return the inverse sine of X.  */
double
DEFUN(__asin, (x), double x)
{
  double abs_x;

  if (__isnan(x)
#ifndef _IEEE_FP
      || __abs(x) > one
#endif
      )
    {
      return infnan(EDOM);
    }

  abs_x = __abs(x);

  if (abs_x <= 0.5)
    return __atan2(x, __sqrt(one - (x * x)));
  else
    {
      double t = one - abs_x;
      return __atan2(x, __sqrt((t + t) - (t * t)));
    }
}

#if (defined(__ELF__) || defined(__GNU_LIBRARY__)) && !defined(_G_NO_STABS)
#include <gnu-stabs.h>
#ifdef weak_alias
weak_alias (__asin, asin);
#endif
#endif
