/* This part here added by roland@prep.ai.mit.edu for the GNU C library.  */
   
#include <ansidecl.h>
#include <math.h>		/* Done first so we can #undef.  */
#include <endian.h>

/*
 * Copyright (c) 1988, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)mathimpl.h	8.1 (Berkeley) 6/4/93
 */

#include <sys/cdefs.h>
#include <math.h>

enum {
  MC_ZERO,		/* 0.0 */
  MC_HALF,		/* 0.5 */
  MC_ONE,		/* 1.0 */
  MC_TWO,		/* 2.0 */
  MC_BIG10,		/* 1e10 */
  MC_BIG18,		/* 1e18 */
  MC_BIG20,		/* 1e20 */
  MC_SMALL9,		/* 1e-9 */
  MC_SMALL10,		/* 1e-10 */
  MC_SMALL18,		/* 1e-18 */
  MC_SMALL19,		/* 1e-19 */
  MC_SMALL20,		/* 1e-20 */
  MC_THRESH,
  MC_PIO4,
  MC_PIO2,
  MC_PI3O4,
  MC_PI,
  MC_PI2,
  MC_SIN0, MC_SIN1, MC_SIN2, MC_SIN3, MC_SIN4, MC_SIN5,
  MC_COS0, MC_COS1, MC_COS2, MC_COS3, MC_COS4, MC_COS5,
  MC_LN2HI,
  MC_LN2LO,
  MC_LNHUGE,
  MC_INVLN2,
  MC_L,
  MC_ATHFHI, MC_ATHFLO,
  MC_AT1FHI, MC_AT1FLO,
  MC_A1, MC_A2, MC_A3, MC_A4, MC_A5, MC_A6,
  MC_A7, MC_A8, MC_A9, MC_A10, MC_A11,
  MC_R2P1HI, MC_R2P1LO,
  MC_SQRT2,
  MC_MLN2HI, MC_MLN2LO,
  MC_LNOVFL,
  MC_P1, MC_P2, MC_P3, MC_P4, MC_P5,
  MC_LNTINY,
  MC_EP1, MC_EP2,
  MC_EQ1, MC_EQ2,
  MC_L1, MC_L2, MC_L3, MC_L4, MC_L5, MC_L6, MC_L7,
  NUM_CONSTS
};

extern const double __math_const[NUM_CONSTS];

#if defined(__GNUC__) && defined(__alpha__)
# define zero	(0.0)	/* aka $f31 */
#else
# define zero	__math_const[ZERO]
#endif

#define half	(__math_const[MC_HALF])
#define one	(__math_const[MC_ONE])
#define two	(__math_const[MC_TWO])
#define big10	(__math_const[MC_BIG10])
#define big18	(__math_const[MC_BIG18])
#define big20	(__math_const[MC_BIG20])
#define small9	(__math_const[MC_SMALL9])
#define small10	(__math_const[MC_SMALL10])
#define small18	(__math_const[MC_SMALL18])
#define small19	(__math_const[MC_SMALL19])
#define small20	(__math_const[MC_SMALL20])
#define ln2hi	(__math_const[MC_LN2HI])
#define ln2lo	(__math_const[MC_LN2LO])
#define lnhuge	(__math_const[MC_LNHUGE])
#define invln2	(__math_const[MC_INVLN2])

#ifdef _IEEE_FP_WITH_INEXACT
# ifdef __GNUC__
   /* pretend x+y is used so the optimizer doesn't get rid of it: */
#  define set_inexact_flag(x,y)	asm volatile ("# %0" :: "f"(x+y));
# else
#  define set_inexact_flag(x,y)	x+y
# endif
#else /* _IEEE_FP_WITH_INEXACT */
#  define set_inexact_flag(x,y)	/* don't bother */
#endif

#ifndef _IEEE_FP
# undef __finite
# define __finite(x)	(1)	/* all numbers are finite if no IEEE fp */
# define __isinf(x)	(0)	/* no numbers are infinite if no IEEE fp */
# define __isnan(x)	(0)	/* all numbers are numbers if no IEEE fp */
#endif

#if defined(__alpha__) && defined(__GNUC__)

extern inline double
__copysign (double value, double sign)
{
    double r;

    asm ("cpys %2,%1,%0" : "f="(r) : "f"(value), "f"(sign));
    return r;
}

extern inline double __abs (double x);

extern inline double
__abs (double x)
{
    double r;

    asm ("cpys $f31,%1,%0" : "f="(r) : "f"(x));
    return r;
}

#else

#define __abs(x)	(__copysign((x), 1.0))

#endif

/*
 * Functions for which there are no prototypes in math.h.  Should
 * they be there??
 */

extern __CONSTVALUE double __sin __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __cos __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __tan __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __asin __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __acos __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __atan __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __atan2 __P((double y, double x)) __CONSTVALUE2;

extern __CONSTVALUE double __sinh __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __cosh __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __tanh __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __asinh __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __acosh __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __atanh __P((double x)) __CONSTVALUE2;

extern __CONSTVALUE double __pow __P((double x, double y)) __CONSTVALUE2;
extern __CONSTVALUE double __sqrt __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __ldexp __P((double x, int exp)) __CONSTVALUE2;
extern __CONSTVALUE double __fabs __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __cabs __P((struct __cabs_complex z)) __CONSTVALUE2;
extern double __z_abs __P((struct __cabs_complex *z));
extern __CONSTVALUE double __hypot __P((double x, double y)) __CONSTVALUE2;
extern __CONSTVALUE double __log __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __log1p __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __log10 __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __exp __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __exp__D __P((double x, double c)) __CONSTVALUE2;
extern __CONSTVALUE double __fmod __P((double x, double y)) __CONSTVALUE2;
extern __CONSTVALUE double __cbrt __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __ceil __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __floor __P((double x)) __CONSTVALUE2;
extern __CONSTVALUE double __lgamma __P((double arg)) __CONSTVALUE2;

extern double __modf __P((double value, double *iptr));
extern double __frexp __P((double value, int *exp));

/*
 * Functions internal to the math package, yet not static.
 */
extern __CONSTVALUE double	__exp__E __P((double x, double c)) __CONSTVALUE2;
extern __CONSTVALUE double	__log__L __P((double z)) __CONSTVALUE2;

struct Double {double a, b;};

extern double __exp__D __P((double, double)) __CONSTVALUE2;
extern struct Double __log__D __P((double)) __CONSTVALUE2;
